@@ -24,14 +24,13 @@
 #include <string.h>
 #include <sys/stat.h>
 
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 
-#include <libmissioncontrol/mc-account.h>
 #include <libmissioncontrol/mc-protocol.h>
 
 #include <libempathy/empathy-utils.h>
+#include <libempathy/empathy-account.h>
 
 #include <libempathy/empathy-irc-network-manager.h>
 #include "empathy-ui-utils.h"
@@ -257,7 +256,7 @@ irc_network_dialog_network_update_buttons (EmpathyIrcNetworkDialog *dialog)
 
     can_remove = TRUE;
     can_move_up = selected > 0;
-    can_move_down = 
+    can_move_down =
       selected < gtk_tree_model_iter_n_children (model, NULL) - 1;
 
     gtk_tree_path_free (path);
@@ -457,7 +456,7 @@ empathy_irc_network_dialog_show (EmpathyIrcNetwork *network,
                                  GtkWidget *parent)
 {
   static EmpathyIrcNetworkDialog *dialog = NULL;
-  GladeXML *glade;
+  GtkBuilder *gui;
   GtkListStore *store;
   GtkCellRenderer *renderer;
   GtkAdjustment *adjustment;
@@ -480,11 +479,9 @@ empathy_irc_network_dialog_show (EmpathyIrcNetwork *network,
   dialog->network = network;
   g_object_ref (dialog->network);
 
-  filename = empathy_file_lookup ("empathy-account-widget-irc.glade",
+  filename = empathy_file_lookup ("empathy-account-widget-irc.ui",
       "libempathy-gtk");
-  glade = empathy_glade_get_file (filename,
-      "irc_network_dialog",
-      NULL,
+  gui = empathy_builder_get_file (filename,
       "irc_network_dialog", &dialog->dialog,
       "button_close", &dialog->button_close,
       "entry_network", &dialog->entry_network,
@@ -518,7 +515,7 @@ empathy_irc_network_dialog_show (EmpathyIrcNetwork *network,
       1, 10, 0);
   renderer = gtk_cell_renderer_spin_new ();
   g_object_set (renderer,
-      "editable", TRUE, 
+      "editable", TRUE,
       "adjustment", adjustment,
       NULL);
   g_signal_connect (renderer, "edited",
@@ -550,7 +547,7 @@ empathy_irc_network_dialog_show (EmpathyIrcNetwork *network,
 
   irc_network_dialog_setup (dialog);
 
-  empathy_glade_connect (glade, dialog,
+  empathy_builder_connect (gui, dialog,
       "irc_network_dialog", "destroy", irc_network_dialog_destroy_cb,
       "button_close", "clicked", irc_network_dialog_close_clicked_cb,
       "entry_network", "focus-out-event", irc_network_dialog_network_focus_cb,
@@ -561,7 +558,7 @@ empathy_irc_network_dialog_show (EmpathyIrcNetwork *network,
       "combobox_charset", "changed", irc_network_dialog_combobox_charset_changed_cb,
       NULL);
 
-  g_object_unref (glade);
+  g_object_unref (gui);
 
   g_object_add_weak_pointer (G_OBJECT (dialog->dialog),
       (gpointer) &dialog);
@@ -575,6 +572,7 @@ empathy_irc_network_dialog_show (EmpathyIrcNetwork *network,
   gtk_window_set_modal (GTK_WINDOW (dialog->dialog), TRUE);
 
   irc_network_dialog_network_update_buttons (dialog);
+  gtk_widget_show_all (dialog->dialog);
 
   return dialog->dialog;
 }
