@@ -100,7 +100,8 @@ chatroom_manager_file_save (EmpathyChatroomManager *manager)
 			continue;
 		}
 
-		account_id = mc_account_get_unique_name (empathy_chatroom_get_account (chatroom));
+		account_id = empathy_account_get_unique_name (
+		  empathy_chatroom_get_account (chatroom));
 
 		node = xmlNewChild (root, NULL, "chatroom", NULL);
 		xmlNewTextChild (node, NULL, "name", empathy_chatroom_get_name (chatroom));
@@ -174,7 +175,7 @@ chatroom_manager_parse_chatroom (EmpathyChatroomManager *manager,
 {
 	EmpathyChatroomManagerPriv *priv;
 	EmpathyChatroom            *chatroom;
-	McAccount                 *account;
+	EmpathyAccount             *account;
 	xmlNodePtr                 child;
 	gchar                     *str;
 	gchar                     *name;
@@ -220,7 +221,7 @@ chatroom_manager_parse_chatroom (EmpathyChatroomManager *manager,
 		xmlFree (str);
 	}
 
-	account = mc_account_lookup (account_id);
+	account = empathy_account_manager_lookup (priv->account_manager, account_id);
 	if (!account) {
 		g_free (name);
 		g_free (room);
@@ -506,6 +507,28 @@ empathy_chatroom_manager_add (EmpathyChatroomManager *manager,
   return FALSE;
 }
 
+static void
+chatroom_manager_remove_link (EmpathyChatroomManager *manager,
+                              GList *l)
+{
+  EmpathyChatroomManagerPriv *priv;
+  EmpathyChatroom *chatroom;
+
+  priv = GET_PRIV (manager);
+
+  chatroom = l->data;
+
+  if (empathy_chatroom_is_favorite (chatroom))
+    reset_save_timeout (manager);
+
+  priv->chatrooms = g_list_delete_link (priv->chatrooms, l);
+
+  g_signal_emit (manager, signals[CHATROOM_REMOVED], 0, chatroom);
+  g_signal_handlers_disconnect_by_func (chatroom, chatroom_changed_cb, manager);
+
+  g_object_unref (chatroom);
+}
+
 void
 empathy_chatroom_manager_remove (EmpathyChatroomManager *manager,
                                  EmpathyChatroom        *chatroom)
@@ -527,15 +550,7 @@ empathy_chatroom_manager_remove (EmpathyChatroomManager *manager,
       if (this_chatroom == chatroom ||
           empathy_chatroom_equal (chatroom, this_chatroom))
         {
-          priv->chatrooms = g_list_delete_link (priv->chatrooms, l);
-          if (empathy_chatroom_is_favorite (chatroom))
-            reset_save_timeout (manager);
-
-          g_signal_emit (manager, signals[CHATROOM_REMOVED], 0, this_chatroom);
-          g_signal_handlers_disconnect_by_func (chatroom, chatroom_changed_cb,
-              manager);
-
-          g_object_unref (this_chatroom);
+          chatroom_manager_remove_link (manager, l);
           break;
         }
     }
@@ -543,21 +558,20 @@ empathy_chatroom_manager_remove (EmpathyChatroomManager *manager,
 
 EmpathyChatroom *
 empathy_chatroom_manager_find (EmpathyChatroomManager *manager,
-                               McAccount *account,
+                               EmpathyAccount *account,
                                const gchar *room)
 {
 	EmpathyChatroomManagerPriv *priv;
 	GList                     *l;
 
 	g_return_val_if_fail (EMPATHY_IS_CHATROOM_MANAGER (manager), NULL);
-	g_return_val_if_fail (MC_IS_ACCOUNT (account), NULL);
 	g_return_val_if_fail (room != NULL, NULL);
 
 	priv = GET_PRIV (manager);
 
 	for (l = priv->chatrooms; l; l = l->next) {
 		EmpathyChatroom *chatroom;
-		McAccount      *this_account;
+		EmpathyAccount *this_account;
 		const gchar    *this_room;
 
 		chatroom = l->data;
@@ -576,7 +590,7 @@ empathy_chatroom_manager_find (EmpathyChatroomManager *manager,
 
 GList *
 empathy_chatroom_manager_get_chatrooms (EmpathyChatroomManager *manager,
-				       McAccount             *account)
+				       EmpathyAccount *account)
 {
 	EmpathyChatroomManagerPriv *priv;
 	GList                     *chatrooms, *l;
@@ -606,7 +620,7 @@ empathy_chatroom_manager_get_chatrooms (EmpathyChatroomManager *manager,
 
 guint
 empathy_chatroom_manager_get_count (EmpathyChatroomManager *manager,
-				   McAccount             *account)
+				   EmpathyAccount *account)
 {
 	EmpathyChatroomManagerPriv *priv;
 	GList                     *l;
@@ -649,13 +663,16 @@ chatroom_manager_chat_destroyed_cb (EmpathyTpChat *chat,
         continue;
 
       empathy_chatroom_set_tp_chat (chatroom, NULL);
+
       if (!empathy_chatroom_is_favorite (chatroom))
         {
           /* Remove the chatroom from the list, unless it's in the list of
            * favourites..
            * FIXME this policy should probably not be in libempathy */
-          empathy_chatroom_manager_remove (manager, chatroom);
+          chatroom_manager_remove_link (manager, l);
         }
+
+      break;
     }
 }
 
@@ -670,7 +687,7 @@ chatroom_manager_observe_channel_cb (EmpathyDispatcher *dispatcher,
   const gchar *roomname;
   GQuark channel_type;
   TpHandleType handle_type;
-  McAccount *account;
+  EmpathyAccount *account;
   TpConnection *connection;
 
   channel_type = empathy_dispatch_operation_get_channel_type_id (operation);
