@@ -15,9 +15,9 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- * 
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ *
  * Authors: Martyn Russell <martyn@imendio.com>
  *          Xavier Claessens <xclaesse@gmail.com>
  */
@@ -28,11 +28,9 @@
 #include <stdlib.h>
 
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 #include <glib/gi18n.h>
 #include <dbus/dbus-glib.h>
 
-#include <libmissioncontrol/mc-account.h>
 #include <libmissioncontrol/mc-profile.h>
 #include <telepathy-glib/util.h>
 
@@ -73,7 +71,8 @@ typedef struct {
 	GtkWidget        *hbox_type;
 	GtkWidget        *button_create;
 	GtkWidget        *button_back;
-	GtkWidget        *checkbutton_register;
+	GtkWidget        *radiobutton_reuse;
+	GtkWidget        *radiobutton_register;
 
 	GtkWidget        *image_type;
 	GtkWidget        *label_name;
@@ -96,7 +95,7 @@ enum {
 };
 
 static void       accounts_dialog_update_account            (EmpathyAccountsDialog    *dialog,
-							     McAccount                *account);
+							     EmpathyAccount           *account);
 static void       accounts_dialog_model_setup               (EmpathyAccountsDialog    *dialog);
 static void       accounts_dialog_model_add_columns         (EmpathyAccountsDialog    *dialog);
 static void       accounts_dialog_name_editing_started_cb   (GtkCellRenderer          *renderer,
@@ -109,19 +108,19 @@ static void       accounts_dialog_model_pixbuf_data_func    (GtkTreeViewColumn
 							     GtkTreeModel             *model,
 							     GtkTreeIter              *iter,
 							     EmpathyAccountsDialog    *dialog);
-static McAccount *accounts_dialog_model_get_selected        (EmpathyAccountsDialog    *dialog);
+static EmpathyAccount *accounts_dialog_model_get_selected        (EmpathyAccountsDialog    *dialog);
 static void       accounts_dialog_model_set_selected        (EmpathyAccountsDialog    *dialog,
-							     McAccount                *account);
+							     EmpathyAccount           *account);
 static gboolean   accounts_dialog_model_remove_selected     (EmpathyAccountsDialog    *dialog);
 static void       accounts_dialog_model_selection_changed   (GtkTreeSelection         *selection,
 							     EmpathyAccountsDialog    *dialog);
 static void       accounts_dialog_add_or_update_account     (EmpathyAccountsDialog    *dialog,
-							     McAccount                *account);
+							     EmpathyAccount           *account);
 static void       accounts_dialog_account_added_cb          (EmpathyAccountManager    *manager,
-							     McAccount                *account,
+							     EmpathyAccount           *account,
 							     EmpathyAccountsDialog    *dialog);
 static void       accounts_dialog_account_removed_cb        (EmpathyAccountManager    *manager,
-							     McAccount                *account,
+							     EmpathyAccount           *account,
 							     EmpathyAccountsDialog    *dialog);
 static gboolean   accounts_dialog_row_changed_foreach       (GtkTreeModel             *model,
 							     GtkTreePath              *path,
@@ -129,7 +128,7 @@ static gboolean   accounts_dialog_row_changed_foreach       (GtkTreeModel
 							     gpointer                  user_data);
 static gboolean   accounts_dialog_flash_connecting_cb       (EmpathyAccountsDialog    *dialog);
 static void       accounts_dialog_connection_changed_cb     (EmpathyAccountManager    *manager,
-							     McAccount                *account,
+							     EmpathyAccount           *account,
 							     TpConnectionStatusReason  reason,
 							     TpConnectionStatus        current,
 							     TpConnectionStatus        previous,
@@ -154,12 +153,12 @@ static void       accounts_dialog_destroy_cb                (GtkWidget
 
 static void
 accounts_dialog_update_name_label (EmpathyAccountsDialog *dialog,
-				   McAccount             *account)
+				   EmpathyAccount        *account)
 {
 	gchar *text;
 
 	text = g_markup_printf_escaped ("<big><b>%s</b></big>",
-			mc_account_get_display_name (account));
+			empathy_account_get_display_name (account));
 	gtk_label_set_markup (GTK_LABEL (dialog->label_name), text);
 
 	g_free (text);
@@ -167,7 +166,7 @@ accounts_dialog_update_name_label (EmpathyAccountsDialog *dialog,
 
 static void
 accounts_dialog_update_account (EmpathyAccountsDialog *dialog,
-				McAccount            *account)
+				EmpathyAccount       *account)
 {
 	McProfile   *profile;
 	const gchar *config_ui;
@@ -184,7 +183,8 @@ accounts_dialog_update_account (EmpathyAccountsDialog *dialog,
 			accounts_dialog_model_select_first (dialog);
 			return;
 		}
-		if (empathy_profile_chooser_n_profiles (dialog->combobox_profile) > 0) {
+		if (empathy_profile_chooser_n_profiles (
+			EMPATHY_PROFILE_CHOOSER (dialog->combobox_profile)) > 0) {
 			/* We have no account configured but we have some
 			 * profiles instsalled. The user obviously wants to add
 			 * an account. Click on the Add button for him. */
@@ -215,12 +215,12 @@ accounts_dialog_update_account (EmpathyAccountsDialog *dialog,
 		dialog->settings_widget = NULL;
 	}
 
-	profile = mc_account_get_profile (account);
+	profile = empathy_account_get_profile (account);
 	config_ui = mc_profile_get_configuration_ui (profile);
 	if (!tp_strdiff (config_ui, "jabber")) {
-		dialog->settings_widget = 
+		dialog->settings_widget =
 			empathy_account_widget_jabber_new (account);
-	} 
+	}
 	else if (!tp_strdiff (config_ui, "msn")) {
 		dialog ->settings_widget =
 			empathy_account_widget_msn_new (account);
@@ -233,11 +233,11 @@ accounts_dialog_update_account (EmpathyAccountsDialog *dialog,
 		dialog->settings_widget =
 			empathy_account_widget_irc_new (account);
 	}
-	else if (!tp_strdiff(config_ui, "icq")) {
+	else if (!tp_strdiff (config_ui, "icq")) {
 		dialog->settings_widget =
 			empathy_account_widget_icq_new (account);
 	}
-	else if (!tp_strdiff(config_ui, "aim")) {
+	else if (!tp_strdiff (config_ui, "aim")) {
 		dialog->settings_widget =
 			empathy_account_widget_aim_new (account);
 	}
@@ -254,12 +254,13 @@ accounts_dialog_update_account (EmpathyAccountsDialog *dialog,
 			empathy_account_widget_groupwise_new (account);
 	}
 	else {
-		dialog->settings_widget = 
+		dialog->settings_widget =
 			empathy_account_widget_generic_new (account);
 	}
 
 	gtk_container_add (GTK_CONTAINER (dialog->alignment_settings),
 			   dialog->settings_widget);
+	gtk_widget_show (dialog->settings_widget);
 
 
 	gtk_image_set_from_icon_name (GTK_IMAGE (dialog->image_type),
@@ -280,10 +281,10 @@ accounts_dialog_model_setup (EmpathyAccountsDialog *dialog)
 	GtkTreeSelection *selection;
 
 	store = gtk_list_store_new (COL_COUNT,
-				    G_TYPE_BOOLEAN,    /* enabled */
-				    G_TYPE_STRING,     /* name */
-				    G_TYPE_UINT,       /* status */
-				    MC_TYPE_ACCOUNT);  /* account */
+				    G_TYPE_BOOLEAN,        /* enabled */
+				    G_TYPE_STRING,         /* name */
+				    G_TYPE_UINT,           /* status */
+				    EMPATHY_TYPE_ACCOUNT); /* account */
 
 	gtk_tree_view_set_model (GTK_TREE_VIEW (dialog->treeview),
 				 GTK_TREE_MODEL (store));
@@ -309,7 +310,7 @@ accounts_dialog_name_edited_cb (GtkCellRendererText   *renderer,
 				gchar                 *new_text,
 				EmpathyAccountsDialog *dialog)
 {
-	McAccount    *account;
+	EmpathyAccount    *account;
 	GtkTreeModel *model;
 	GtkTreePath  *treepath;
 	GtkTreeIter   iter;
@@ -331,7 +332,7 @@ accounts_dialog_name_edited_cb (GtkCellRendererText   *renderer,
 			    -1);
 	gtk_tree_path_free (treepath);
 
-	mc_account_set_display_name (account, new_text);
+	empathy_account_set_display_name (account, new_text);
 	g_object_unref (account);
 }
 
@@ -340,7 +341,7 @@ accounts_dialog_enable_toggled_cb (GtkCellRendererToggle *cell_renderer,
 				   gchar                 *path,
 				   EmpathyAccountsDialog *dialog)
 {
-	McAccount    *account;
+	EmpathyAccount    *account;
 	GtkTreeModel *model;
 	GtkTreePath  *treepath;
 	GtkTreeIter   iter;
@@ -354,11 +355,11 @@ accounts_dialog_enable_toggled_cb (GtkCellRendererToggle *cell_renderer,
 			    -1);
 	gtk_tree_path_free (treepath);
 
-	enabled = mc_account_is_enabled (account);
-	mc_account_set_enabled (account, !enabled);
+	enabled = empathy_account_is_enabled (account);
+	empathy_account_set_enabled (account, !enabled);
 
 	DEBUG ("%s account %s", enabled ? "Disabled" : "Enable",
-		mc_account_get_display_name(account));
+		empathy_account_get_display_name (account));
 
 	g_object_unref (account);
 }
@@ -395,7 +396,7 @@ accounts_dialog_model_add_columns (EmpathyAccountsDialog *dialog)
 	g_signal_connect (cell, "toggled",
 			  G_CALLBACK (accounts_dialog_enable_toggled_cb),
 			  dialog);
-	
+
 	/* Account column */
 	column = gtk_tree_view_column_new ();
 	gtk_tree_view_column_set_title (column, _("Accounts"));
@@ -415,6 +416,7 @@ accounts_dialog_model_add_columns (EmpathyAccountsDialog *dialog)
 	cell = gtk_cell_renderer_text_new ();
 	g_object_set (cell,
 		      "ellipsize", PANGO_ELLIPSIZE_END,
+		      "width-chars", 25,
 		      "editable", TRUE,
 		      NULL);
 	gtk_tree_view_column_pack_start (column, cell, TRUE);
@@ -438,7 +440,7 @@ accounts_dialog_model_select_first (EmpathyAccountsDialog *dialog)
 	/* select first */
 	view = GTK_TREE_VIEW (dialog->treeview);
 	model = gtk_tree_view_get_model (view);
-	
+
 	if (gtk_tree_model_get_iter_first (model, &iter)) {
 		selection = gtk_tree_view_get_selection (view);
 		gtk_tree_selection_select_iter (selection, &iter);
@@ -454,7 +456,7 @@ accounts_dialog_model_pixbuf_data_func (GtkTreeViewColumn    *tree_column,
 					GtkTreeIter          *iter,
 					EmpathyAccountsDialog *dialog)
 {
-	McAccount          *account;
+	EmpathyAccount     *account;
 	const gchar        *icon_name;
 	GdkPixbuf          *pixbuf;
 	TpConnectionStatus  status;
@@ -469,7 +471,7 @@ accounts_dialog_model_pixbuf_data_func (GtkTreeViewColumn    *tree_column,
 
 	if (pixbuf) {
 		if (status == TP_CONNECTION_STATUS_DISCONNECTED ||
-		    (status == TP_CONNECTION_STATUS_CONNECTING && 
+		    (status == TP_CONNECTION_STATUS_CONNECTING &&
 		     !dialog->connecting_show)) {
 			GdkPixbuf *modded_pixbuf;
 
@@ -501,14 +503,14 @@ accounts_dialog_model_pixbuf_data_func (GtkTreeViewColumn    *tree_column,
 
 static gboolean
 accounts_dialog_get_account_iter (EmpathyAccountsDialog *dialog,
-				 McAccount             *account,
+				 EmpathyAccount        *account,
 				 GtkTreeIter           *iter)
 {
 	GtkTreeView      *view;
 	GtkTreeSelection *selection;
 	GtkTreeModel     *model;
 	gboolean          ok;
-	
+
 	/* Update the status in the model */
 	view = GTK_TREE_VIEW (dialog->treeview);
 	selection = gtk_tree_view_get_selection (view);
@@ -517,7 +519,7 @@ accounts_dialog_get_account_iter (EmpathyAccountsDialog *dialog,
 	for (ok = gtk_tree_model_get_iter_first (model, iter);
 	     ok;
 	     ok = gtk_tree_model_iter_next (model, iter)) {
-		McAccount *this_account;
+		EmpathyAccount *this_account;
 		gboolean   equal;
 
 		gtk_tree_model_get (model, iter,
@@ -535,14 +537,14 @@ accounts_dialog_get_account_iter (EmpathyAccountsDialog *dialog,
 	return FALSE;
 }
 
-static McAccount *
+static EmpathyAccount *
 accounts_dialog_model_get_selected (EmpathyAccountsDialog *dialog)
 {
 	GtkTreeView      *view;
 	GtkTreeModel     *model;
 	GtkTreeSelection *selection;
 	GtkTreeIter       iter;
-	McAccount        *account;
+	EmpathyAccount   *account;
 
 	view = GTK_TREE_VIEW (dialog->treeview);
 	selection = gtk_tree_view_get_selection (view);
@@ -558,7 +560,7 @@ accounts_dialog_model_get_selected (EmpathyAccountsDialog *dialog)
 
 static void
 accounts_dialog_model_set_selected (EmpathyAccountsDialog *dialog,
-				    McAccount             *account)
+				    EmpathyAccount        *account)
 {
 	GtkTreeSelection *selection;
 	GtkTreeIter       iter;
@@ -591,7 +593,7 @@ static void
 accounts_dialog_model_selection_changed (GtkTreeSelection     *selection,
 					 EmpathyAccountsDialog *dialog)
 {
-	McAccount    *account;
+	EmpathyAccount    *account;
 	GtkTreeModel *model;
 	GtkTreeIter   iter;
 	gboolean      is_selection;
@@ -608,7 +610,7 @@ accounts_dialog_model_selection_changed (GtkTreeSelection     *selection,
 
 static void
 accounts_dialog_add_or_update_account (EmpathyAccountsDialog *dialog,
-				       McAccount             *account)
+				       EmpathyAccount        *account)
 {
 	GtkTreeModel       *model;
 	GtkTreeIter         iter;
@@ -617,9 +619,9 @@ accounts_dialog_add_or_update_account (EmpathyAccountsDialog *dialog,
 	gboolean            enabled;
 
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (dialog->treeview));
-	status = mission_control_get_connection_status (dialog->mc, account, NULL);
-	name = mc_account_get_display_name (account);
-	enabled = mc_account_is_enabled (account);
+	g_object_get (account, "status", &status, NULL);
+	name = empathy_account_get_display_name (account);
+	enabled = empathy_account_is_enabled (account);
 
 	if (!accounts_dialog_get_account_iter (dialog, account, &iter)) {
 		DEBUG ("Adding new account");
@@ -643,7 +645,7 @@ accounts_dialog_add_or_update_account (EmpathyAccountsDialog *dialog,
 
 static void
 accounts_dialog_account_added_cb (EmpathyAccountManager *manager,
-				  McAccount *account,
+				  EmpathyAccount *account,
 				  EmpathyAccountsDialog *dialog)
 {
 	const gchar *current_name;
@@ -653,25 +655,25 @@ accounts_dialog_account_added_cb (EmpathyAccountManager *manager,
 
 	/* Change the display name to "%s (%s)" % (protocol, account).
 	 *  - The protocol is the display name of the profile.
-	 *  - The account should be the normalized name of the McAccount but
+	 *  - The account should be the normalized name of the EmpathyAccount but
 	 *    it's not set until first connection, so we get the "account"
 	 *    parameter for CM that have it. */
-	current_name = mc_account_get_display_name (account);
-	mc_account_get_param_string (account, "account", &account_param);
+	current_name = empathy_account_get_display_name (account);
+	account_param = empathy_account_get_param_string (account, "account");
 	if (!EMP_STR_EMPTY (account_param)) {
 		McProfile   *profile;
 		const gchar *profile_name;
 		gchar       *new_name;
 
-		profile = mc_account_get_profile (account);
+		profile = empathy_account_get_profile (account);
 		profile_name = mc_profile_get_display_name (profile);
 		new_name = g_strdup_printf ("%s (%s)", profile_name,
 					    account_param);
 
 		DEBUG ("Setting new display name for account %s: '%s'",
-		       mc_account_get_unique_name (account), new_name);
+		       empathy_account_get_unique_name (account), new_name);
 
-		mc_account_set_display_name (account, new_name);
+		empathy_account_set_display_name (account, new_name);
 		g_free (new_name);
 		g_object_unref (profile);
 	} else {
@@ -682,7 +684,7 @@ accounts_dialog_account_added_cb (EmpathyAccountManager *manager,
 
 static void
 accounts_dialog_account_removed_cb (EmpathyAccountManager *manager,
-				    McAccount            *account,
+				    EmpathyAccount       *account,
 				    EmpathyAccountsDialog *dialog)
 {
 
@@ -719,7 +721,7 @@ accounts_dialog_flash_connecting_cb (EmpathyAccountsDialog *dialog)
 
 static void
 accounts_dialog_connection_changed_cb     (EmpathyAccountManager    *manager,
-					   McAccount                *account,
+					   EmpathyAccount           *account,
 					   TpConnectionStatusReason  reason,
 					   TpConnectionStatus        current,
 					   TpConnectionStatus        previous,
@@ -728,7 +730,7 @@ accounts_dialog_connection_changed_cb     (EmpathyAccountManager    *manager,
 	GtkTreeModel *model;
 	GtkTreeIter   iter;
 	gboolean      found;
-	
+
 	/* Update the status in the model */
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (dialog->treeview));
 
@@ -760,7 +762,7 @@ accounts_dialog_connection_changed_cb     (EmpathyAccountManager    *manager,
 
 static void
 enable_or_disable_account (EmpathyAccountsDialog *dialog,
-			   McAccount *account,
+			   EmpathyAccount *account,
 			   gboolean enabled)
 {
 	GtkTreeModel *model;
@@ -770,7 +772,7 @@ enable_or_disable_account (EmpathyAccountsDialog *dialog,
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (dialog->treeview));
 
 	DEBUG ("Account %s is now %s",
-		mc_account_get_display_name (account),
+		empathy_account_get_display_name (account),
 		enabled ? "enabled" : "disabled");
 
 	if (accounts_dialog_get_account_iter (dialog, account, &iter)) {
@@ -782,7 +784,7 @@ enable_or_disable_account (EmpathyAccountsDialog *dialog,
 
 static void
 accounts_dialog_account_disabled_cb (EmpathyAccountManager *manager,
-				     McAccount             *account,
+				     EmpathyAccount        *account,
 				     EmpathyAccountsDialog *dialog)
 {
 	enable_or_disable_account (dialog, account, FALSE);
@@ -790,7 +792,7 @@ accounts_dialog_account_disabled_cb (EmpathyAccountManager *manager,
 
 static void
 accounts_dialog_account_enabled_cb (EmpathyAccountManager *manager,
-				    McAccount             *account,
+				    EmpathyAccount        *account,
 				    EmpathyAccountsDialog *dialog)
 {
 	enable_or_disable_account (dialog, account, TRUE);
@@ -798,10 +800,10 @@ accounts_dialog_account_enabled_cb (EmpathyAccountManager *manager,
 
 static void
 accounts_dialog_account_changed_cb (EmpathyAccountManager *manager,
-				    McAccount             *account,
+				    EmpathyAccount        *account,
 				    EmpathyAccountsDialog  *dialog)
 {
-	McAccount *selected_account;
+	EmpathyAccount *selected_account;
 
 	accounts_dialog_add_or_update_account (dialog, account);
 	selected_account = accounts_dialog_model_get_selected (dialog);
@@ -815,33 +817,35 @@ accounts_dialog_button_create_clicked_cb (GtkWidget             *button,
 					  EmpathyAccountsDialog  *dialog)
 {
 	McProfile *profile;
-	McAccount *account;
+	EmpathyAccount *account;
 	gchar     *str;
 	McProfileCapabilityFlags cap;
 
-	profile = empathy_profile_chooser_get_selected (dialog->combobox_profile);
+	profile = empathy_profile_chooser_dup_selected (
+	    EMPATHY_PROFILE_CHOOSER (dialog->combobox_profile));
 
 	/* Create account */
-	account = mc_account_create (profile);
+	account = empathy_account_manager_create (dialog->account_manager, profile);
 	if (account == NULL) {
 		/* We can't display an error to the user as MC doesn't give us
 		 * any clue about the reason of the failure... */
+		g_object_unref (profile);
 		return;
 	}
 
 	/* To translator: %s is the protocol name */
 	str = g_strdup_printf (_("New %s account"),
 			       mc_profile_get_display_name (profile));
-	mc_account_set_display_name (account, str);
+	empathy_account_set_display_name (account, str);
 	g_free (str);
 
 	cap = mc_profile_get_capabilities (profile);
 	if (cap & MC_PROFILE_CAPABILITY_REGISTRATION_UI) {
 		gboolean active;
 
-		active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->checkbutton_register));
-		if (!active) {
-			mc_account_set_param_boolean (account, "register", TRUE);
+		active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->radiobutton_register));
+		if (active) {
+			empathy_account_set_param_boolean (account, "register", TRUE);
 		}
 	}
 
@@ -856,7 +860,7 @@ static void
 accounts_dialog_button_back_clicked_cb (GtkWidget             *button,
 					EmpathyAccountsDialog  *dialog)
 {
-	McAccount *account;
+	EmpathyAccount *account;
 
 	account = accounts_dialog_model_get_selected (dialog);
 	accounts_dialog_update_account (dialog, account);
@@ -869,13 +873,16 @@ accounts_dialog_profile_changed_cb (GtkWidget             *widget,
 	McProfile *profile;
 	McProfileCapabilityFlags cap;
 
-	profile = empathy_profile_chooser_get_selected (dialog->combobox_profile);
+	profile = empathy_profile_chooser_dup_selected (
+	    EMPATHY_PROFILE_CHOOSER (dialog->combobox_profile));
 	cap = mc_profile_get_capabilities (profile);
 
 	if (cap & MC_PROFILE_CAPABILITY_REGISTRATION_UI) {
-		gtk_widget_show (dialog->checkbutton_register);
+		gtk_widget_show (dialog->radiobutton_register);
+		gtk_widget_show (dialog->radiobutton_reuse);
 	} else {
-		gtk_widget_hide (dialog->checkbutton_register);
+		gtk_widget_hide (dialog->radiobutton_register);
+		gtk_widget_hide (dialog->radiobutton_reuse);
 	}
 	g_object_unref (profile);
 }
@@ -906,8 +913,8 @@ accounts_dialog_button_add_clicked_cb (GtkWidget             *button,
 		gtk_widget_hide (dialog->button_back);
 	}
 
-	accounts_dialog_profile_changed_cb (dialog->checkbutton_register, dialog);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->checkbutton_register),
+	accounts_dialog_profile_changed_cb (dialog->radiobutton_register, dialog);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->radiobutton_reuse),
 				      TRUE);
 	gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->combobox_profile), 0);
 	gtk_widget_grab_focus (dialog->combobox_profile);
@@ -924,13 +931,13 @@ static void
 accounts_dialog_button_remove_clicked_cb (GtkWidget            *button,
 					  EmpathyAccountsDialog *dialog)
 {
-	McAccount *account;
+	EmpathyAccount *account;
 	GtkWidget *message_dialog;
 	gint       res;
 
 	account = accounts_dialog_model_get_selected (dialog);
 
-	if (!mc_account_is_complete (account)) {
+	if (!empathy_account_is_valid (account)) {
 		accounts_dialog_model_remove_selected (dialog);
 		accounts_dialog_model_select_first (dialog);
 		return;
@@ -942,7 +949,7 @@ accounts_dialog_button_remove_clicked_cb (GtkWidget            *button,
 		 GTK_BUTTONS_NONE,
 		 _("You are about to remove your %s account!\n"
 		   "Are you sure you want to proceed?"),
-		 mc_account_get_display_name (account));
+		 empathy_account_get_display_name (account));
 
 	gtk_message_dialog_format_secondary_text
 		(GTK_MESSAGE_DIALOG (message_dialog),
@@ -953,17 +960,17 @@ accounts_dialog_button_remove_clicked_cb (GtkWidget            *button,
 		   "they will still be available."));
 
 	gtk_dialog_add_button (GTK_DIALOG (message_dialog),
-			       GTK_STOCK_CANCEL, 
+			       GTK_STOCK_CANCEL,
 			       GTK_RESPONSE_NO);
 	gtk_dialog_add_button (GTK_DIALOG (message_dialog),
-			       GTK_STOCK_REMOVE, 
+			       GTK_STOCK_REMOVE,
 			       GTK_RESPONSE_YES);
 
 	gtk_widget_show (message_dialog);
 	res = gtk_dialog_run (GTK_DIALOG (message_dialog));
 
 	if (res == GTK_RESPONSE_YES) {
-		mc_account_delete (account);
+		empathy_account_manager_remove (dialog->account_manager, account);
 		accounts_dialog_model_select_first (dialog);
 	}
 	gtk_widget_destroy (message_dialog);
@@ -1013,15 +1020,15 @@ accounts_dialog_destroy_cb (GtkWidget            *widget,
 					      dialog);
 
 	/* Delete incomplete accounts */
-	accounts = mc_accounts_list ();
+	accounts = empathy_account_manager_dup_accounts (dialog->account_manager);
 	for (l = accounts; l; l = l->next) {
-		McAccount *account;
+		EmpathyAccount *account;
 
 		account = l->data;
-		if (!mc_account_is_complete (account)) {
+		if (!empathy_account_is_valid (account)) {
 			/* FIXME: Warn the user the account is not complete
 			 *        and is going to be removed. */
-			mc_account_delete (account);
+			empathy_account_manager_remove (dialog->account_manager, account);
 		}
 
 		g_object_unref (account);
@@ -1034,16 +1041,16 @@ accounts_dialog_destroy_cb (GtkWidget            *widget,
 
 	g_object_unref (dialog->account_manager);
 	g_object_unref (dialog->mc);
-	
+
 	g_free (dialog);
 }
 
 GtkWidget *
 empathy_accounts_dialog_show (GtkWindow *parent,
-			      McAccount *selected_account)
+			      EmpathyAccount *selected_account)
 {
 	static EmpathyAccountsDialog *dialog = NULL;
-	GladeXML                     *glade;
+	GtkBuilder                   *gui;
 	gchar                        *filename;
 	GList                        *accounts, *l;
 	gboolean                      import_asked;
@@ -1055,11 +1062,9 @@ empathy_accounts_dialog_show (GtkWindow *parent,
 
 	dialog = g_new0 (EmpathyAccountsDialog, 1);
 
-	filename = empathy_file_lookup ("empathy-accounts-dialog.glade",
+	filename = empathy_file_lookup ("empathy-accounts-dialog.ui",
 					"src");
-	glade = empathy_glade_get_file (filename,
-				       "accounts_dialog",
-				       NULL,
+	gui = empathy_builder_get_file (filename,
 				       "accounts_dialog", &dialog->window,
 				       "vbox_details", &dialog->vbox_details,
 				       "frame_no_profile", &dialog->frame_no_profile,
@@ -1069,7 +1074,8 @@ empathy_accounts_dialog_show (GtkWindow *parent,
 				       "hbox_type", &dialog->hbox_type,
 				       "button_create", &dialog->button_create,
 				       "button_back", &dialog->button_back,
-				       "checkbutton_register", &dialog->checkbutton_register,
+				       "radiobutton_reuse", &dialog->radiobutton_reuse,
+				       "radiobutton_register", &dialog->radiobutton_register,
 				       "image_type", &dialog->image_type,
 				       "label_name", &dialog->label_name,
 				       "button_add", &dialog->button_add,
@@ -1078,8 +1084,7 @@ empathy_accounts_dialog_show (GtkWindow *parent,
 				       NULL);
 	g_free (filename);
 
-	empathy_glade_connect (glade,
-			      dialog,
+	empathy_builder_connect (gui, dialog,
 			      "accounts_dialog", "destroy", accounts_dialog_destroy_cb,
 			      "accounts_dialog", "response", accounts_dialog_response_cb,
 			      "button_create", "clicked", accounts_dialog_button_create_clicked_cb,
@@ -1092,7 +1097,7 @@ empathy_accounts_dialog_show (GtkWindow *parent,
 
 	g_object_add_weak_pointer (G_OBJECT (dialog->window), (gpointer) &dialog);
 
-	g_object_unref (glade);
+	g_object_unref (gui);
 
 	/* Create profile chooser */
 	dialog->combobox_profile = empathy_profile_chooser_new ();
@@ -1130,7 +1135,7 @@ empathy_accounts_dialog_show (GtkWindow *parent,
 	accounts_dialog_model_setup (dialog);
 
 	/* Add existing accounts */
-	accounts = mc_accounts_list ();
+	accounts = empathy_account_manager_dup_accounts (dialog->account_manager);
 	for (l = accounts; l; l = l->next) {
 		accounts_dialog_add_or_update_account (dialog, l->data);
 		g_object_unref (l->data);
@@ -1153,11 +1158,17 @@ empathy_accounts_dialog_show (GtkWindow *parent,
 	empathy_conf_get_bool (empathy_conf_get (),
 			       EMPATHY_PREFS_IMPORT_ASKED, &import_asked);
 
-	if (!import_asked) {
-		empathy_conf_set_bool (empathy_conf_get (),
-				       EMPATHY_PREFS_IMPORT_ASKED, TRUE);
-		empathy_import_dialog_show (GTK_WINDOW (dialog->window),
-					    FALSE);
+
+	if (empathy_import_dialog_accounts_to_import ()) {
+
+		if (!import_asked) {
+			empathy_conf_set_bool (empathy_conf_get (),
+					       EMPATHY_PREFS_IMPORT_ASKED, TRUE);
+			empathy_import_dialog_show (GTK_WINDOW (dialog->window),
+						    FALSE);
+		}
+	} else {
+		gtk_widget_set_sensitive (dialog->button_import, FALSE);
 	}
 
 	return dialog->window;
